
# トラブルシューティング


## 動作計画が得られない

### 動作姿勢に問題がある

動作計画時にロボットの姿勢に問題がある場合は動作計画が得られません．

- ターゲットの姿勢がロボットの機構として可能な姿勢ではない
  - 位置が近すぎる・遠すぎる
	- 方向がロボットの構造的に無理がある
- ターゲットの姿勢間の動作計画軌道内に不可能な姿勢が存在する

#### 【対策】姿勢を変更する・姿勢の経由点を追加する

ターゲット姿勢を変更したり，
無理のない姿勢の経由点を追加するなどの対策を行ってみてください．

### 計算がタイムアウトする

動作計画の計算に要する時間が長くなる場合にはタイムアウトして結果が得られない場合があります．

#### 【対策】動作計画のプランナを指定する

動作計画のプランナが「デフォルト」の設定の場合に計算時間が長くなることがあります．
その場合にはプランナをデフォルト以外のもの，
例えば `RRTConnectkConfigDefault` に変更すると短時間で計算が終了する場合があります．

```python
group.set_planner_id( "RRTConnectkConfigDefault" )
```

#### 【対策】動作計画の計算タイムアウト設定を長くする

動作計画に多くの計算が必要な場合は設定時間内に計算が終わらない場合があります．
その場合は `set_planning_time()` でタイムアウトの設定を変更します．

```python
group.set_planning_time( 30.0 )
```


## 動作計画が実行されない

### 動作計画軌道上に障害物が存在

動作計画の解が得られても実行時のチェックで障害物との干渉が発見されると動作が実行されません．

#### 【対策】動作の再計画やターゲット姿勢などの変更

動作の再計画やターゲット姿勢の変更を行ってください．


## tf が取得できない

### 画像処理ノードとの関係上の問題

画像処理には多くの計算リソースを使用することが多かったり，
姿勢などのタイミングで画像処理の対象がカメラに映っていなかったりと，
動作計画をする瞬間は適切に tf の発行がされていない場合があります．

#### 【対策】tf 変換の試行回数を多くする

tf 変換の試行を長く，多くして結果を得られる機会を増やしてみてください．

本チュートリアルで準備している tork_moveit_tools にある
`get_current_target_pose()` を使用する場合は引数の `timeout` を長めにします．

```python
pose_target = get_current_target_pose( tgt_frame_id, planning_frame_id, 5.0 )
```

### 同期が取れていない - 実機ロボットの場合

シミュレータを使用している場合は大体は同一の PC 上で実行されていることが多いので
tf の同期に問題が出ることはほぼありませんが，
実機ロボットとの通信下で様々なノードを実行している場合は制御コンピュータや
クライアントPCの間で同期が取れずに tf 変換がされない場合があります．

#### 【対策】NTP サーバと同期をとる

NTP サーバとの同期をとる作業を数回行ってください．

```
$ sudo ntpdate -bv <ntp_server_address>
```

目安として時間のずれが 10 [msec] 以下になるようにします．

NTP サーバはロボットである場合や他のサーバである場合があります．
ロボット管理者に NTP サーバの設定について問い合わせてください．


## Gazebo の問題

<a id="gazebo-start-not-show-robot"></a>
### Gazebo を起動してもロボットが表示されない

#### 【対策】PC で初めて Gazebo を起動した場合は「しばらく待つ」

作業を行う PC で初めて Gazebo を起動する場合は
モデルの読み込みなどに時間がかかることがあり，
単に時間がかかっているだけということがあります．

#### 【対策】ROS 環境設定を確認する

ROS の環境設定に問題がある場合があります．
ターミナルで ROS の環境設定がされているか確認します．

```
$ env | grep ROS
```

正常な場合の出力例

```
ROS_ROOT=/opt/ros/<$ROS_DISTRO>/share/ros
ROS_PACKAGE_PATH=/opt/ros/<$ROS_DISTRO>/share:/opt/ros/<$ROS_DISTRO>/stacks
ROS_MASTER_URI=http://localhost:11311
ROSLISP_PACKAGE_DIRECTORIES=
ROS_DISTRO=<$ROS_DISTRO>
ROS_ETC_DIR=/opt/ros/<$ROS_DISTRO>/etc/ros
```

ROS の環境設定に問題がある場合は設定し直します．

```
$ source /opt/ros/<$ROS_DISTRO>/setup.bash
```

#### 【対策】全ての ROS ノードを終了して再起動

何度か Gazebo などを利用して終了を繰り返していると
ROS のノードが終了しきれていないなどの理由で
新しい ROS ノードの起動に不具合が生じることがあります．

たまに Gazebo が正常に起動しなくなるような場合はこれに相当することが多いです．

Gazebo を Ctrl-C で終了して他の ROS ノードも全て終了していることを確認してから
Gazebo を再起動してみてください．



<!-- EOF -->
